@@ -7,13 +7,14 @@ main ()
 {
   struct S { char s; } v;
   v.s = 47;
-  int a = (int) v.s;
-  int b = (27005061 + (a + 680455));
-  int c = ((1207142401 * (((8 * b) + 9483541) - 230968044)) + 469069442);
-  if (c != 1676211843)
+  unsigned int a = (unsigned int) v.s;
+  unsigned int b = (27005061 + (a + 680455));
+  unsigned int c
+    = ((1207142401u * (((8u * b) + 9483541u) - 230968044u)) + 469069442u);
+  if (c != 1676211843u)
     __builtin_abort ();
   return 0;
 }
 
-/* { dg-final { scan-tree-dump-times "b \\\* 8" 1 "gimple" } } */
+/* { dg-final { scan-tree-dump-times "b \\\* 1067204616" 1 "gimple" } } */
 /* { dg-final { scan-tree-dump-not "abort" "optimized" } } */
